# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/dialogs/login/browser_login.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_BrowserLogin(object):
    def setupUi(self, BrowserLogin):
        BrowserLogin.setObjectName("BrowserLogin")
        BrowserLogin.resize(182, 210)
        BrowserLogin.setWindowTitle("BrowserLogin")
        self.browser_layout = QtWidgets.QGridLayout(BrowserLogin)
        self.browser_layout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.browser_layout.setObjectName("browser_layout")
        self.link_layout = QtWidgets.QHBoxLayout()
        self.link_layout.setObjectName("link_layout")
        self.link_text = QtWidgets.QLineEdit(BrowserLogin)
        self.link_text.setText("")
        self.link_text.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.link_text.setReadOnly(True)
        self.link_text.setPlaceholderText("")
        self.link_text.setObjectName("link_text")
        self.link_layout.addWidget(self.link_text)
        self.copy_button = QtWidgets.QPushButton(BrowserLogin)
        self.copy_button.setText("")
        self.copy_button.setObjectName("copy_button")
        self.link_layout.addWidget(self.copy_button)
        self.browser_layout.addLayout(self.link_layout, 2, 0, 1, 2)
        spacerItem = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.browser_layout.addItem(spacerItem, 4, 0, 1, 2)
        self.open_button = QtWidgets.QPushButton(BrowserLogin)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.open_button.sizePolicy().hasHeightForWidth())
        self.open_button.setSizePolicy(sizePolicy)
        self.open_button.setObjectName("open_button")
        self.browser_layout.addWidget(self.open_button, 1, 0, 1, 1)
        self.sid_layout = QtWidgets.QHBoxLayout()
        self.sid_layout.setObjectName("sid_layout")
        self.browser_layout.addLayout(self.sid_layout, 1, 1, 1, 1)
        self.title_label = QtWidgets.QLabel(BrowserLogin)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.title_label.setFont(font)
        self.title_label.setObjectName("title_label")
        self.browser_layout.addWidget(self.title_label, 0, 0, 1, 2)
        self.info_label = QtWidgets.QLabel(BrowserLogin)
        font = QtGui.QFont()
        font.setItalic(True)
        self.info_label.setFont(font)
        self.info_label.setWordWrap(True)
        self.info_label.setObjectName("info_label")
        self.browser_layout.addWidget(self.info_label, 5, 0, 1, 2)
        self.status_label = QtWidgets.QLabel(BrowserLogin)
        self.status_label.setText("")
        self.status_label.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.status_label.setObjectName("status_label")
        self.browser_layout.addWidget(self.status_label, 3, 0, 1, 2)

        self.retranslateUi(BrowserLogin)
        QtCore.QMetaObject.connectSlotsByName(BrowserLogin)

    def retranslateUi(self, BrowserLogin):
        _translate = QtCore.QCoreApplication.translate
        self.open_button.setText(_translate("BrowserLogin", "Open Browser"))
        self.title_label.setText(_translate("BrowserLogin", "Login through browser"))
        self.info_label.setText(_translate("BrowserLogin", "Click the button to open the login page in a browser or copy the link and paste it in a browser. After logging in, copy the <b><code>authorizationCode</code></b> in the input above."))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    BrowserLogin = QtWidgets.QWidget()
    ui = Ui_BrowserLogin()
    ui.setupUi(BrowserLogin)
    BrowserLogin.show()
    sys.exit(app.exec_())
